/* LoongArch specific backend library.
 *
 * Copyright (C) 2022 OpenAnolis community LoongArch SIG.
 * 
 * Author: Liwei Ge <geliwei@openanolis.org>
 *
 */

/*
   This file is free software; you can redistribute it and/or modify
   it under the terms of either

     * the GNU Lesser General Public License as published by the Free
       Software Foundation; either version 3 of the License, or (at
       your option) any later version

   or

     * the GNU General Public License as published by the Free
       Software Foundation; either version 2 of the License, or (at
       your option) any later version

   or both in parallel, as here.

   elfutils is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received copies of the GNU General Public License and
   the GNU Lesser General Public License along with this program.  If
   not, see <http://www.gnu.org/licenses/>.
*/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#define BACKEND		loongarch_
#define RELOC_PREFIX	R_LARCH_
#include "libebl_CPU.h"

/* This defines the common reloc hooks based on loongarch_reloc.def.  */
#include "common-reloc.c"


Ebl *
loongarch_init (Elf *elf __attribute__ ((unused)),
	  GElf_Half machine __attribute__ ((unused)),
	  Ebl *eh)
{
  loongarch_init_reloc (eh);

  HOOK (eh, machine_flag_check);
  HOOK (eh, check_special_symbol);
  HOOK (eh, reloc_simple_type);
  HOOK (eh, return_value_location);
  HOOK (eh, set_initial_registers_tid);
  HOOK (eh, register_info);
  HOOK (eh, abi_cfi);
  HOOK (eh, unwind);

  /* 
   This is FIRST_PSEUDO_REGISTER in GCC, since DWARF_FRAME_REGISTERS
   is not explicitly defined.

   Number of hardware registers. We have:
     - 32 integer registers
     - 32 floating point registers
     - 8 condition code registers
     - 2 fake registers:
        - ARG_POINTER_REGNUM
        - FRAME_POINTER_REGNUM  
  */
  eh->frame_nregs = 74;

  return eh;
}
