/* LoongArch specific backend library.
 *
 * Copyright (C) 2022 OpenAnolis community LoongArch SIG.
 * 
 * Author: Liwei Ge <geliwei@openanolis.org>
 *
 */

/*
   This file is free software; you can redistribute it and/or modify
   it under the terms of either

     * the GNU Lesser General Public License as published by the Free
       Software Foundation; either version 3 of the License, or (at
       your option) any later version

   or

     * the GNU General Public License as published by the Free
       Software Foundation; either version 2 of the License, or (at
       your option) any later version

   or both in parallel, as here.

   elfutils is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received copies of the GNU General Public License and
   the GNU Lesser General Public License along with this program.  If
   not, see <http://www.gnu.org/licenses/>.
*/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#define BACKEND loongarch_
#define RA_REG 3
#define SP_REG 3
#define FP_REG 22

#define RA_OFFSET 8
#define SP_OFFSET 16

#include "libebl_CPU.h"


bool
loongarch_unwind (Ebl *ebl __attribute__ ((unused)), Dwarf_Addr pc __attribute__ ((unused)),
                 ebl_tid_registers_t *setfunc, ebl_tid_registers_get_t *getfunc,
                 ebl_pid_memory_read_t *readfunc, void *arg,
                 bool *signal_framep __attribute__ ((unused)))
{

  Dwarf_Word ra;
  if (!getfunc(RA_REG, 1, &ra, arg))
    return false;
  /* set next frame */
  setfunc(-1, 1, &ra, arg);

  Dwarf_Word fp, sp;
  /* read from regs */
  if (!getfunc(FP_REG, 1, &fp, arg))
    fp = 0;
  if (!getfunc(SP_REG, 1, &sp, arg))
    sp = 0;

  /* read from frame */
  Dwarf_Word prev_fp;
  if (!readfunc(fp, &prev_fp, arg))
    prev_fp = 0;
  Dwarf_Word new_ra;
  if (!readfunc(fp + RA_OFFSET, &new_ra, arg))
    return false;
  Dwarf_Word new_sp = fp + SP_OFFSET;

  /* set regs */
  setfunc(FP_REG, 1, &prev_fp, arg);
  setfunc(RA_REG, 1, &new_ra, arg);
  setfunc(SP_REG, 1, &new_sp, arg);

  return fp == 0 || new_sp > sp;
}
